// XEOL.C v1.0
// (c)1996, Jonas Skendelis. All rights reserved.
// Any portion of this text can be used as sample, only.
// No commercial use. ANY!
// Any portion of source or documentation text or executables
// cannot be changed without permision of the author.

// Description:
// To port textual files among Unix, DOS and MAC's
// The program converts line breaks, eliminates backspaced chars
//         and erases trailing blanks in lines
// Very fast!



// Author: Jonas Skendelis

// History:
// Creating date: 1996.01.17.
// Release date: 1996.01.17.
// Revisee date: 1996.01.23.  v1.0A
// Revise date: 1996.01.28.   v1.0B

// Notes: The version for Unix'es is going, soon.
// The new version with encoding Lithuanians is going soon, also.
// If You need them, call the author of XEOL directly!

// Legend: I had the experience to port text from Unix to PC.
// What happens You may to see if try to port MAN texts.
// They are not readable!
// I tried to solve this problem in various ways:
// 1. I wrote the script for AWK (It exist in versions of HP-UX and SCO Unix)
// 2. I wrote the small program in C on HP-UX
// 3. I wrote the macros in Word for PC...

//    And now, I wrote the XEOL program for PC!
//    Enjoy it,
//    Jonas Skendelis


// Start of code

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dos.h>
#include <direct.h>
#include <fcntl.h>
#include <io.h>

#include <malloc.h>
#include <ctype.h>

#include "xeol-dos.h"

static char far *readbuf = NULL;
static char far *outputbuf = NULL;

static unsigned int numread = 0;
static unsigned int outquant = 0;
static unsigned int bytestoread = 0;
static unsigned int bytesafterread = 0;

static int hSource = -1; // Input file handle
static int hTarget = -1; // Output file handle

static BOOL bWasCRLF = FALSE;

static WORD wSourceOS = defSourceOS;
static WORD wTargetOS = defTargetOS;

static char ffpat[32] = "*.txt"; // Pattern to search files.
static char cOutExt[] = "new";   // The extent for new file name.
static char cAltExt[] = "txt";   // The extent for new file name (alternate)

static BOOL OutByte(char cbyte)
{ WORD xx;
  if (outquant >= maxbufsize)
    { if ( _dos_write(hTarget, outputbuf, outquant, &xx) == -1) goto write_err;
      if (xx != outquant)
        { write_err: printf("\nWriting error\n");
          return TRUE;
        }
      outquant = 0;
    }

  outputbuf[outquant++] = cbyte;
  return FALSE;
} // end OutByte

static BOOL IsWhiteSpace(char c)
{ if ((c == BLANK_CHAR) || (c == TAB_CHAR)) return TRUE;
  return FALSE;
}

static void EatTrailingWhiteSpace()
{ while (outquant > 0)
   { if (IsWhiteSpace(outputbuf[outquant-1])) --outquant;
     else break;
   }
} // end EatTrailingWhiteSpace

static BOOL PurgeOutput()
{ WORD xx;
  if ( _dos_write(hTarget, outputbuf, outquant, &xx) == -1) goto write_err;
  if (xx != outquant)
    { write_err: printf("\nWriting error\n");
      return TRUE;
    }
  return (BOOL)(outquant = 0);
} // end PurgeOutput

static int TryAllocBuffs(unsigned int bsize)
{ int retval = 0;

  if ( (readbuf = (char far *)_fmalloc(bsize)) == NULL) goto alloc_err;

  if ( (outputbuf = (char far *)_fmalloc(bsize+0x100)) == NULL)
  { _ffree(readbuf);
     alloc_err:
     printf ("\nNot enough conventional memory to IO buffers.\n");
     retval = 8;
   }
  return retval;
} // end TryAllocBuffs

static void freeBuffs()
 { if (readbuf != NULL)   { _ffree(readbuf); readbuf = NULL; }
   if (outputbuf != NULL) { _ffree(outputbuf);  outputbuf = NULL; }
 } // end freeBuffs

static BOOL OutputLineBreak()
{ if (((wTargetOS & MAC_OS_CASE) != 0) || ((wTargetOS & MSDOS_CASE) != 0))
     if (OutByte(CR_CHAR)) return TRUE;

 if (((wTargetOS & ANY_UNIX_CASE) != 0) || ((wTargetOS & MSDOS_CASE) != 0))
    if (OutByte(LF_CHAR)) return TRUE;
 return FALSE;
} // end OutputLineBreak


static int ToCutPattern(char *ffn, char *ptn, int l_ptn)
{ int k = strlen(ffn);
  int L = 0;

  while ((--k) >= 0)
    { if ((ffn[k] == '\\') || (ffn[k] == ':'))  break;
      ++L;
    }

 if (L > l_ptn) return -1;
 ++k;
 strcpy(ptn, ffn+k);
 return k;
} // end ToCutPattern

static int FindFNExt(char *fn)
{ int k = strlen(fn);
  int L = 0;

  while ((--k) >= 0)
    { if (fn[k] == '.') return (k+1);
      if ((fn[k] == '\\') || (fn[k] == ':'))  break;
    }

  return -1;
} // end FindFNExt


// The analyse of the program's parameters

static BOOL bWasHelp = FALSE;

static BOOL bWasFrom = FALSE;
static BOOL bWasTo   = FALSE;
static BOOL bWasOutput = FALSE;
static BOOL bWasAlternate = FALSE;

static BOOL bWasLith = FALSE;
static BOOL bWasfCP  = FALSE;
static BOOL bWasrCP  = FALSE;

// The tails of keys
static char cFrom[]    = "rom";
static char cTo[]      = "o";
static char cOut[]     = "ut";
static char cOutput[]  = "utput";
static char cAlt[]     = "lt";
static char cAlternate[]  = "lternate";

static char cLith[] = "ithuanian";
static char cLithAlt[] = "th";
static char cfCP[] = "ourcecp";
static char crCP[] = "esultcp";

static char *cOSes[] = {"unix", "dos", "mac", "os2", NULL};
static char *cCPs[] = {"kbl", "rim", "unix", "mac",
                       "770", "771", "772", "774",  NULL };

static char cIncParm[] = "\nNot allowed value of the field %s:%s";
static char cLongExt[] = "\nThe length of file extention in %s parameter cannot be greater than 3";
static char cNoOnce[] = "\nThe parameter '%s:%s' is used more than once";

// Coding tables for Lithuanians
static char CP_KBL[] = "";
static char CP_RST[] = "";
static char CP_772_774[] = "";
static char CP_UNIX_BALTIC_RIM[] = "";
static char CP_MAC_Respublic[] = "";

static char *pSourceCP = defSourceCP;
static char *pTargetCP = defTargetCP;

static int FindInList(char *clist[], char *str)
{ char *ptr = clist[0];
  int index = 0;

  while (ptr != NULL)
    { if (stricmp(ptr, str) == 0) return index;
      ptr = clist[++index];
    }
  return -1;
} // end FindInList

static char cUFROM[] = "FROM";

static BOOL IsFromKey(char *mparamP2)
{ int k, L = 0;

  if (mparamP2[L] == ':')    { }
  else if (strnicmp(mparamP2, cFrom, L = strlen(cFrom)) != 0) return FALSE;
  ++L;
  if ((k = FindInList(cOSes, mparamP2+L)) != -1)
    { if (bWasFrom)
        { printf (cNoOnce, cUFROM, mparamP2+L);
          return FALSE;
        }

      bWasFrom = TRUE;
      switch (k)
        { case 0:  wSourceOS = ANY_UNIX_CASE;
                   break;
          case 2:  wSourceOS = MAC_OS_CASE;
                   break;
          case 3:  wSourceOS = OS2_CASE;
                   break;
          case 1:
          default: wSourceOS = MSDOS_CASE;
                   break;
        }
      return TRUE;
    }
  else  printf (cIncParm, cUFROM, mparamP2+L);
  return FALSE;
} // end IsFromKey

static char cUTO[] = "TO";

static BOOL IsToKey(char *mparamP2)
{ int k, L = 0;

  if (mparamP2[L] == ':') { }
  else if (strnicmp(mparamP2, cTo, L = strlen(cTo)) != 0) return FALSE;
  ++L;
  if ((k = FindInList(cOSes, mparamP2+L)) != -1)
    { if (bWasTo)
        { printf (cNoOnce, cUTO, mparamP2+L);
          return FALSE;
        }

      bWasTo = TRUE;
      switch (k)
        { case 0:  wTargetOS = ANY_UNIX_CASE;
                   break;
          case 2:  wTargetOS = MAC_OS_CASE;
                   break;
          case 3:  wTargetOS = OS2_CASE;
                   break;
          case 1:
          default: wTargetOS = MSDOS_CASE;
                   break;
        }
      return TRUE;
    }
  else  printf (cIncParm, cUTO, mparamP2+L);

  return FALSE;
} // end IsToKey

static char cUOUTPUT[] = "OUTPUT";

static BOOL IsOutputKey (char *mparamP2)
{ int L = 0;

  if (mparamP2[L] == ':')    { }
  else
    { if (strnicmp(mparamP2, cOut, L = strlen(cOut)) == 0)
        if (mparamP2[L] != ':')
           if (strnicmp(mparamP2, cOutput, L = strlen(cOutput)) != 0)
                                                        return FALSE;
    }
  ++L;

  if (bWasOutput)
     { printf (cNoOnce, cUOUTPUT, mparamP2+L);
       return FALSE;
     }

  if (strlen(mparamP2+L) > 3)
     { printf (cLongExt, cUOUTPUT);
       return FALSE;
     }
  strcpy (cOutExt, mparamP2+L);

  bWasOutput = TRUE;
  return TRUE;
} // end IsOutputKey

static char cUALTERNATE[] = "ALTERNATE";

static BOOL IsAlternateKey (char *mparamP2)
{ int L = 0;

  if (mparamP2[L] == ':')    { }
  else
    { if (strnicmp(mparamP2, cAlt, L = strlen(cAlt)) != 0)
        if (mparamP2[L] != ':')
            if (strnicmp(mparamP2, cAlternate, L = strlen(cAlternate)) != 0)
                                                               return FALSE;
    }
  ++L;

  if (bWasAlternate)
     { printf (cNoOnce, cUALTERNATE, mparamP2+L);
       return FALSE;
     }

  if (strlen(mparamP2+L) > 3)
     { printf (cLongExt, cUALTERNATE);
       return FALSE;
     }

  strcpy (cAltExt, mparamP2+L);
  bWasAlternate = TRUE;
  return TRUE;
} // end IsAlternateKey

static BOOL IsLithuanian(char *mparamP2)
{ if ((stricmp(cLith, mparamP2) == 0) || (stricmp(cLithAlt, mparamP2) == 0))
    { if (bWasLith)
        {

        }
      bWasLith = TRUE;
    }
  return FALSE;
} // end IsLithuanian

static BOOL IsSourceCP(char *mparamP2)
{ int k, L = 0;

  if (mparamP2[L] == ':') { }
  else if (strnicmp(mparamP2, cfCP, L = strlen(cfCP)) != 0)  return FALSE;
  ++L;

  if ((k = FindInList(cCPs, mparamP2+L)) != -1)
    { if (bWasfCP)
        {

        }

     bWasfCP = TRUE;

     switch (k)
       { case 0:
         case 5:   pSourceCP = CP_KBL;
                   break;
         case 1:
         case 2:   pSourceCP = CP_UNIX_BALTIC_RIM;
                   break;
         case 3:   pSourceCP = CP_MAC_Respublic;
                   break;

         case 4:   pSourceCP = CP_RST;
                   break;
         case 6:
         case 7:   pSourceCP = CP_772_774;
                   break;
         default:
                  break;
       }

    }
  else printf (cIncParm, "SourceCP", mparamP2+L);

  return FALSE;
}

static BOOL IsResultCP(char *mparamP2)
{ int k, L = 0;

  if (mparamP2[L] == ':')   { }
  else if (strnicmp(mparamP2, crCP, L = strlen(crCP)) != 0)   return FALSE;
  ++L;

  if ((k = FindInList(cCPs, mparamP2+L)) != -1)
    { if (bWasrCP)
        {

        }

     bWasrCP = TRUE;

     switch (k)
       { case 0:
         case 5:
                   pTargetCP = CP_KBL;
                   break;
         case 1:
         case 2:   pTargetCP = CP_UNIX_BALTIC_RIM;
                   break;
         case 3:   pTargetCP = CP_MAC_Respublic;
                   break;

         case 4:   pTargetCP = CP_RST;
                   break;
         case 6:
         case 7:   pTargetCP = CP_772_774;
                   break;
         default:
                  break;
       }

    }
  else printf (cIncParm, "SourceCP", mparamP2+L);

  return FALSE;
} // end IsResultCP

static BOOL WhatKey(char *mparam)
{ char c = (char)tolower(mparam[1]);

  if (c)
     { switch (c)
         { case 'f': if (IsFromKey(mparam+2))      return TRUE;
                     break;
           case 't': if (IsToKey(mparam+2))        return TRUE;
                     break;
           case 'o': if (IsOutputKey(mparam+2))    return TRUE;
                     break;
           case 'a': if (IsAlternateKey(mparam+2)) return TRUE;
                     break;

           case '?': if (mparam[2] == '\0')
                       { bWasHelp = TRUE;
                         return TRUE;
                       }
                     break;

           case 'h': if (stricmp (mparam+2, "elp") == 0)
                       { bWasHelp = TRUE;
                         return TRUE;
                       }
                     break;

           case 'l': if (IsLithuanian(mparam+2))   return TRUE;
                     break;
           case 's': if (IsSourceCP(mparam+2))     return TRUE;
                     break;
           case 'r': if (IsResultCP(mparam+2))     return TRUE;
                     break;
           default:
                     break;
         }
     }

  printf ("\nThe parameter '%s' is not recognized", mparam);
  return FALSE;
} // end WhatKey

void SetDefLangCP()
{ if      (wSourceOS & ANY_UNIX_CASE)   pSourceCP = defUNIXCP;
  else if ((wSourceOS & MSDOS_CASE) ||
          (wSourceOS & OS2_CASE))       pSourceCP = defDOSCP;
  else if (wSourceOS & MAC_OS_CASE)     pSourceCP = defMACCP;
  else                                  pSourceCP = defSourceCP;

  if (wTargetOS & ANY_UNIX_CASE)        pTargetCP = defUNIXCP;
  else if ((wTargetOS & MSDOS_CASE) ||
          (wTargetOS & OS2_CASE))       pTargetCP = defDOSCP;
  else if (wTargetOS & MAC_OS_CASE)     pTargetCP = defMACCP;
  else                                  pTargetCP = defTargetCP;

} // end SetDefLangCP

BOOL CheckCP()
{

  return TRUE;
}

void OutputHelp()
{ printf ("\nThe use of XEOL:\n"
  "\tXEOL [-f[rom]:OS] [-t[o]:OS] [-o[ut[put]]:extention] [-h[elp]] [-?] fn_pattern"
  "\n\nHere:   [...] mean that the object can omitted;"
  "\n\t'from' and 'to' parameters define the platform;"
  "\n\tOS - one of DOS, UNIX, OS2 or MAC;"
  "\n\textention - the extension of output file, example: '-o:txt';"
  "\n\tfn_pattern defines the path and pattern to seach files.\n");

} // end OutputHelp

// Main program entry
int main(int argc, char *argv[])
{ int k;
  unsigned int count;
  BOOL   bBreak;
  struct find_t ff;
  char   outfn[66];

  printf ("XEOL v1.0 To port Textual files among platforms\n");
  printf ("(c)1996, Jonas Skendelis. All rights reserved. Not for commercial use.\n");

  bBreak = FALSE;

  count = 1;
  while (argc > 1)
    { // patikrinti ir is kelio isskirti sablona

      if ((argv[count][0] == '/') || (argv[count][0] == '-'))
         { // Rakto analize
           if (WhatKey(argv[count]) == FALSE)
             { OutputHelp();
               return 4;
             }
         }
      else
       { if (bBreak)
           { // jau buvo Pattern
             printf ("\nOnly one parameter is posible.\n");
             OutputHelp();
             return 0;
           }

         k = ToCutPattern(argv[count], ffpat, 31);
         bBreak = TRUE;
         if (k > 0)
           { if (argv[count][k-1] == '\\')  --k;
             argv[count][k] = '\0';

             if ((argv[count][1] == ':') && (isalpha(argv[count][0]) != 0))
               { int ndrives;
                 _dos_setdrive(tolower(argv[count][0])-'a'+1, &ndrives);
               }

             if (chdir (argv[count]))
               { printf ("\nCan't change to specified dir %s\n", argv[1]);
                 return 0;
               }
           }
         else if (k < 0)
             { printf ("\nSpecified too long search pattern\n");
               return 0;
             }
       }
      -- argc; count++;
    }  // arg check

  bBreak = TRUE;
  if (bWasLith)
    if ((bWasfCP == FALSE) && (bWasrCP == FALSE))
      { bBreak = FALSE;
        SetDefLangCP();
      }

  if (bWasHelp) OutputHelp();

  if (bBreak) if ((bWasfCP != FALSE) || (bWasrCP != FALSE))
     { // Check correctness of CP compatibility with platforms

       if (!CheckCP())
         {
         }
     }

  if ((k = TryAllocBuffs(maxbufsize))) return k;

  if (_dos_findfirst(ffpat, _A_NORMAL | _A_RDONLY, &ff))
     { printf ("\nNo such files in directory\n"); return 0;
     }

  do { bBreak = FALSE;
       printf ("\nProcessing '%s' file ..", ff.name);
       if (_dos_open(ff.name, 0, /* nesicackinsiu */
                      &hSource) != - 1)
         { // Suformuoti isejimo failo varda
           strcpy(outfn, ff.name);  bBreak = FALSE;

           k = FindFNExt(outfn);

           if (k == -1)
             {
              k = strlen(outfn); *(WORD *)(outfn+k) = '.';
               ++k;
             }

           if (stricmp(outfn+k, cOutExt) == 0)
                strcpy(outfn+k, cAltExt);
           else strcpy(outfn+k, cOutExt);

           if (_dos_creat(outfn, 0, &hTarget) != -1)
             {
              setmode (hTarget, O_BINARY);
              bytestoread = bytesafterread =  maxbufsize;
              numread = 0; outquant = 0; bWasCRLF = FALSE;

               while ((numread+bytestoread) == maxbufsize)
                 { putchar('.');
                   if (_dos_read(hSource, readbuf, maxbufsize,
                        &bytesafterread) == -1)
                     { printf("\nError reading the file %s\n", ff.name);
                       bBreak = TRUE; goto wri_tail;
                     }
                    numread += bytesafterread;

                    for (count = 0; count < numread; count++)
                      { if (readbuf[count] == 8)
                          { if (wSourceOS & ANY_UNIX_CASE)
                              { // Doesn not to eat the line breaks
                                if ((outquant != 0) && (bWasCRLF == FALSE))
                                  --outquant;
                                else
                                  { // Such situation can be at file begin, only.
                                    if (OutByte(readbuf[count]))
                                      { err_wri_tail:
                                        printf ("\nError writing to file:%s", outfn);
                                        bBreak = TRUE;
                                        goto wri_tail;
                                      }
                                  }
                              }
                            else  if (OutByte(readbuf[count])) goto err_wri_tail;
                            bWasCRLF = FALSE;
                          }
                        else
                          { if (readbuf[count] == LF_CHAR)
                              { if ((wSourceOS & MAC_OS_CASE) == 0)
                                  { EatTrailingWhiteSpace();
                                    if (OutputLineBreak()) goto err_wri_tail;
                                    bWasCRLF = TRUE;
                                  }
                              }
                            else
                              if (readbuf[count] == CR_CHAR)
                                { if (wSourceOS & MAC_OS_CASE)
                                    { EatTrailingWhiteSpace();
                                      if (OutputLineBreak()) goto err_wri_tail;
                                      bWasCRLF = TRUE;
                                    }
                                }
                            else
                              { // Any other byte is sending to output
                                if (OutByte(readbuf[count]))
                                            goto err_wri_tail;
                                bWasCRLF = FALSE;
                              }
                          }
                      } // end for
                    bytestoread = numread; numread = 0;
                 } // end while

               if (PurgeOutput()) goto err_wri_tail;

               wri_tail: _dos_close(hTarget);
             } // if creat
           else
             { printf ("\nError creating file %s", outfn);
               bBreak = TRUE;
             }
           _dos_close(hSource);
         } // if open
       else
         { printf ("\nError opening file %s", ff.name);
           bBreak = TRUE;
         }

       if (bBreak) printf ("\nFile %s was not processed succesfully", ff.name);
     } while (_dos_findnext(&ff) == 0);

  freeBuffs();
  printf ("\nAll done.\n");
  return 0;
} // end main
