/* XEOL.C v1.5U
   (c)1996, Jonas Skendelis. All rights reserved.
   Any portion of this source text can be used as sample, only.
   No commercial use. ANY!
   No use in the military and other power structures. ANY!
   Any portion of source or documentation text or executables
   cannot be changed without permision of the author.

   Description:
   To port textual files among Unix, DOS and MAC's
   The program converts line breaks, eliminates backspaced chars
           and erases trailing blanks in lines
   Very fast!

   Version 1.5U does the conversion of Lithuanian & Cyrrilic chars.

   Author: Jonas Skendelis

   History:
   Creating date:      1996.01.17.
   Release date:       1996.01.17.
   v1.5U beta release date: 1996.01.29.
   v1.5U release date: 1996.02.17.

   Note: The new version for MS-DOS is going soon, also.
   The AWK scripts & Word macros will be added.
   If You need them, call the author of XEOL directly!

   Legend: I had the experience to port text from Unix to PC.
   What happens You may to see if try to port MAN texts.
   They are not readable!
   I tried to solve this problem in various ways:
   1. I wrote the script for AWK (It exist in versions of HP-UX and SCO Unix)
   2. I wrote the small program in C on HP-UX
   3. I wrote the macros in Word for PC...

      Then     I wrote the XEOL program for PC! There was v1.0A
      And now, I wrote the common XEOL version for PC & Unix!

      Enjoy it,
      Jonas Skendelis
*/

/* Start of code */

#ifdef _DOS_CASE
  #include <io.h>
#endif

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <malloc.h>
#include <ctype.h>
#include <errno.h>

#ifdef _DOS_CASE
#include <sys\types.h>
#include <sys\stat.h>
#else
#define O_BINARY 0
#ifdef _CASE_OLD_UNIX
#define UU_RDONLY 0
#endif
#include <sys/types.h>
#include <sys/stat.h>

/* This two functions is extension of standart library */
int stricmp(str1, str2) char *str1; char *str2;
{ while ((*str1) && (tolower((int)*str1) == tolower((int)*str2)))
   { ++str1; ++str2; }

  if (*str1)   return (tolower((int)*str1) - tolower((int)*str2));
  if (*str2)   return -1;
  return 0;
} /* end stricmp  */

int strnicmp(str1, str2, N) char *str1; char *str2; int N;
{ int K = 0;

  while ((K < N) && (*str1) && (*str2) &&
                    (tolower((int)*str1) == tolower((int)*str2)))
   { ++str1; ++str2; ++K; }

  if (K < N)
  { if (*str1)
       if (*str2) return (tolower((int)*str1) - tolower((int)*str2));
       else       return 1;

    if (*str2)   return -1;
  }
  return 0;
} /* end strnicmp  */

#endif

#include "xeol.h"

static char *readbuf = NULL;
static char *outputbuf = NULL;

static unsigned int numread = 0;
static unsigned int outquant = 0;
static unsigned int bytestoread = 0;
static unsigned int bytesafterread = 0;

static int hSource = -1; /* Input file handle */
static int hTarget = -1; /* Output file handle */

static BOOL bWasCRLF = FALSE;

static WORD wSourceOS = defSourceOS;
static WORD wTargetOS = defTargetOS;

static char WRI_ERR[] = "\nWriting error.\n";


static BOOL OutByte(cbyte) char cbyte;
{ if (outquant >= maxbufsize)
    { if (write(hTarget, outputbuf, outquant) != outquant)
        { printf(WRI_ERR);
          return TRUE;
        }
      outquant = 0;
    }

  outputbuf[outquant++] = cbyte;
  return FALSE;
} /* end OutByte */

static BOOL IsWhiteSpace(c) char c;
{ if ((c == BLANK_CHAR) || (c == TAB_CHAR)) return TRUE;
  return FALSE;
} /* end IsWhiteSpace */

static void EatTrailingWhiteSpace()
{ while (outquant > 0)
   { if (IsWhiteSpace(outputbuf[outquant-1])) --outquant;
     else break;
   }
} /* end EatTrailingWhiteSpace */

static BOOL PurgeOutput()
{  if (write(hTarget, outputbuf, outquant) == -1)
    { printf(WRI_ERR);
      return TRUE;
    }
  return (BOOL)(outquant = 0);
} /* end PurgeOutput */

static int TryAllocBuffs(bsize) unsigned int bsize;
{ int retval = 0;

  if ( (readbuf = (char *)malloc(bsize)) == NULL) goto alloc_err;

  if ( (outputbuf = (char *)malloc(bsize+0x100)) == NULL)
  { free(readbuf);
     alloc_err:
     printf ("\nNot enough conventional memory to IO buffers.\n");
     retval = 8;
   }
  return retval;
} /* end TryAllocBuffs */

static void freeBuffs()
 { if (readbuf != NULL)   { free(readbuf); readbuf = NULL; }
   if (outputbuf != NULL) { free(outputbuf);  outputbuf = NULL; }
 } /* end freeBuffs */

static BOOL OutputLineBreak()
{ if (((wTargetOS & MAC_OS_CASE) != 0) || ((wTargetOS & MSDOS_CASE) != 0))
     if (OutByte(CR_CHAR)) return TRUE;

 if (((wTargetOS & ANY_UNIX_CASE) != 0) || ((wTargetOS & MSDOS_CASE) != 0))
    if (OutByte(LF_CHAR)) return TRUE;
 return FALSE;
} /* end OutputLineBreak */

/* The analyse of the program's parameters */

static BOOL bWasHelp = FALSE;

static BOOL bWasFrom = FALSE;
static BOOL bWasTo   = FALSE;


static BOOL bWasS772  = FALSE;
static BOOL bWasT772  = FALSE;
static BOOL bWasLith = FALSE;
static BOOL bWasfCP  = FALSE;
static BOOL bWasrCP  = FALSE;
static BOOL bWasCyr  = FALSE;

static BOOL bWasFull = TRUE; /* To skip all symbols */
static BOOL bWasNoTrail = FALSE;


/* The tails of keys */
static char cFrom[]    = "rom";
static char cTo[]      = "o";

static char cLith[] = "ithuanian";
static char cLithAlt[] = "th";

static char cCyr[] = "yrrilic";
static char cCyrAlt[] = "yr";

static char cfCP[] = "ourcecp";
static char crCP[] = "esultcp";
static char tgCP[] = "targetcp";


static char cFull[] = "ull";
static char cNoTrail[] = "otrail";

static char *cOSes[] = {"unix", "dos", "mac", "os2", NULL};
static char *cCPs[] = {"kbl", "rim", "unix", "mac",
                       "770", "771", "772", "774",
                       "spectrum", "mixed", "lema", "tilde",
                       "sigma", "diplomat", NULL };

static char cIncParm[] = "\nNot allowed value of the field %s:%s";
static char cLongExt[] = "\nThe length of file extention in %s parameter cannot be greater than 3";
static char cNoOnce[] = "\nThe parameter '%s:%s' is used more than once";
static char cNoOnce0[] = "\nThe parameter '%s' is used more than once";

/* Coding tables for Lithuanians */
static unsigned char CP_KBL[] =
     {221, 223, 241, 243, 245, 247, 249, 251, 253,
      220, 222, 240, 242, 244, 246, 248, 250, 252, 0};
static unsigned char CP_RST[] =
       "";
static unsigned char CP_772_774[] =
       {208, 209, 210, 211, 212, 213, 214, 215, 216,
        181, 182, 183, 184, 189, 190, 198, 199, 207, 0};

static unsigned char CP_UNIX_BALTIC_RIM[] =
       {0xe0,0xe8,0xe6,0xeb,0xe1,0xf0,0xf8,0xfb,0xfe,
        0xc0,0xc8,0xc6,0xcb,0xc1,0xd0,0xd8,0xdb,0xde, 0};

static unsigned char WIN_LSPEC0[]=
       {224, 230, 232, 233, 236, 246, 249, 250, 251,
        192, 198, 200, 201, 204, 214, 217, 218, 219, 0};

static unsigned char WIN_LSPEC_MIX[] =
       {177, 178, 179, 180, 181, 184, 186, 188, 191,
        161, 185, 163, 176, 165, 168, 170, 172, 175, 0};

static unsigned char CP_MAC_Respublic[] =
     {140, 141,171, 182, 94, 167, 172, 198, 189,
      129, 130, 228, 235, 233, 234, 232, 239, 243, 0};

static unsigned char LTH_Diplomat[] =
     {145, 146, 148, 147, 149, 150, 152, 151, 153,
      161, 162, 164, 163, 165, 166, 168, 167, 169, 0};

static unsigned char  WIN_LEMA[] =
      {0xE1,0xE3,0xEC,0xF2,0xED,0x9A,0xF9,0xFA,0xF5,
       0xC1,0xC3,0xCC,0xD2,0xCD,0x8A,0xD9,0xDA,0xD5,
       0xFD, 0xDD, /* LEMOS DURNYSTS: y ir Y */
       0};

static unsigned char WIN_TILDE[] =
      {227, 224, 234, 233, 237, 248, 251, 250, 253,
       195, 192, 202, 201, 205, 216, 219, 218, 221, 0 };

static  unsigned char  WIN_SIGMA[] =
      {177, 232, 234, 236, 231, 185, 249, 254, 190,
       161, 200, 202, 204, 199, 169, 217, 222, 174, 0 };

static  unsigned char DOS_RUS[] =
                 ""
                 ""
                 ""
                 "";

static unsigned WIN_RUS[] =
  {224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,
   240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,
   192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,
   208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223, 0};


static BOOL bNeedConv = FALSE;
static BOOL bNeedCyrConv = FALSE;

static char *pSourceCP = defSourceCP;
static char *pTargetCP = defTargetCP;
static char *pCyrSourceCP = defCyrSourceCP;
static char *pCyrTargetCP = defCyrTargetCP;

static int FindInList(clist, str) char **clist; char *str;
{ char *ptr = clist[0];
  int index = 0;

  while (ptr != NULL)
    { if (stricmp(ptr, str) == 0) return index;
      ptr = clist[++index];
    }
  return -1;
} /* end FindInList */

static char cUFROM[] = "FROM";

static BOOL IsFromKey(mparamP2) char *mparamP2;
{ int k, L = 0;

  if (mparamP2[L] == ':')    { }
  else if (strnicmp(mparamP2, cFrom, L = strlen(cFrom)) != 0) return FALSE;
  ++L;
  if ((k = FindInList(cOSes, mparamP2+L)) != -1)
    { if (bWasFrom)
        { printf (cNoOnce, cUFROM, mparamP2+L);
          return FALSE;
        }

      bWasFrom = TRUE;
      switch (k)
        { case 0:  wSourceOS = ANY_UNIX_CASE;
                   break;
          case 2:  wSourceOS = MAC_OS_CASE;
                   break;
          case 3:  wSourceOS = OS2_CASE;
                   break;
          case 1:
          default: wSourceOS = MSDOS_CASE;
                   break;
        }
      return TRUE;
    }
  else  printf (cIncParm, cUFROM, mparamP2+L);
  return FALSE;
} /* end IsFromKey */

static char cUTO[] = "TO";

static BOOL IsToKey(mparamP2) char *mparamP2;
{ int k, L = 0;

  if (mparamP2[L] == ':') { }
  else if (strnicmp(mparamP2, cTo, L = strlen(cTo)) != 0) return FALSE;
  ++L;
  if ((k = FindInList(cOSes, mparamP2+L)) != -1)
    { if (bWasTo)
        { printf (cNoOnce, cUTO, mparamP2+L);
          return FALSE;
        }

      bWasTo = TRUE;
      switch (k)
        { case 0:  wTargetOS = ANY_UNIX_CASE;
                   break;
          case 2:  wTargetOS = MAC_OS_CASE;
                   break;
          case 3:  wTargetOS = OS2_CASE;
                   break;
          case 1:
          default: wTargetOS = MSDOS_CASE;
                   break;
        }
      return TRUE;
    }
  else  printf (cIncParm, cUTO, mparamP2+L);

  return FALSE;
} /* end IsToKey */


static BOOL IsFull(mparamP2) char *mparamP2;
{ if (stricmp(cFull, mparamP2) == 0)
    { if (bWasFull == FALSE)
        { printf (cNoOnce0, "FULL");
          return FALSE;

        }
      bWasFull = FALSE;
      return TRUE;
    }
  return FALSE;
} /* end IsFull */

static BOOL IsNoTrail(mparamP2) char *mparamP2;
{ if (stricmp(cNoTrail, mparamP2) == 0)
    { if (bWasNoTrail)
        { printf (cNoOnce0, "NOTRAIL");
          return FALSE;

        }
      bWasNoTrail = TRUE;
      return TRUE;
    }
  return FALSE;
} /* end IsFull */


static char cLITH[] = "LITHUANIAN";

static BOOL IsLithuanian(mparamP2) char *mparamP2;
{ if ((stricmp(cLith, mparamP2) == 0) || (stricmp(cLithAlt, mparamP2) == 0))
    { if (bWasLith)
        { printf (cNoOnce0, cLITH);
          return FALSE;

        }
      bWasLith = TRUE;
      return TRUE;
    }
  return FALSE;
} /* end IsLithuanian */

static BOOL IsCyrillic(mparamP2) char *mparamP2;
{ if ((stricmp(cCyr, mparamP2) == 0) || (stricmp(cCyrAlt, mparamP2) == 0))
    { if (bWasCyr)
        { printf (cNoOnce0, "CYRRILIC");
          return FALSE;
        }
      bWasCyr = TRUE;
      return TRUE;
    }
  return FALSE;
} /* end IsCyrrilic */


static char cSOURCE[] = "SOURCECP";

static BOOL IsSourceCP(mparamP2) char *mparamP2;
{ int k, L = 0;

  if (mparamP2[L] == ':') { }
  else if (strnicmp(mparamP2, cfCP, L = strlen(cfCP)) != 0)  return FALSE;
  ++L;

  if ((k = FindInList(cCPs, mparamP2+L)) != -1)
    { if (bWasfCP)
        { printf (cNoOnce, cSOURCE, mparamP2+L);
          return FALSE;
        }

     bWasfCP = TRUE;

     switch (k)
       { case 0:
         case 5:   pSourceCP = CP_KBL;
                   break;
         case 1:
         case 2:   pSourceCP = CP_UNIX_BALTIC_RIM;
                   break;
         case 3:   pSourceCP = CP_MAC_Respublic;
                   break;

         case 4:   pSourceCP = CP_RST;
                   break;

         case 6:   bWasS772 = TRUE;
         case 7:   pSourceCP = CP_772_774;
                   break;
         case 8:   pSourceCP = WIN_LSPEC0;
                   break;
         case 9:   pSourceCP = WIN_LSPEC_MIX;
                   break;
         case 10:  pSourceCP = WIN_LEMA;
                   break;
         case 11:  pSourceCP = WIN_TILDE;
                   break;
         case 12:  pSourceCP = WIN_SIGMA;
                   break;
         case 13:  pSourceCP = LTH_Diplomat;
                   break;
         default:
                  break;
       }
     return TRUE;
    }
  else printf (cIncParm, cSOURCE, mparamP2+L);

  return FALSE;
} /* end IsSourceCP */


static char cTARGET[] = "TARGETCP";

static BOOL IsResultCP(mparamP2, bAlt) char *mparamP2; BOOL bAlt;
{ int k, L = 0;

  if (!bAlt)
    { if (mparamP2[L] == ':')   { }
      else if (strnicmp(mparamP2, crCP, L = strlen(crCP)) != 0) return FALSE;
    }
  else  if (strnicmp(mparamP2, tgCP, L = strlen(tgCP)) != 0) return FALSE;

  ++L;

  if ((k = FindInList(cCPs, mparamP2+L)) != -1)
    { if (bWasrCP)
        { printf (cNoOnce, cTARGET, mparamP2+L);
          return FALSE;

        }

     bWasrCP = TRUE;

     switch (k)
       { case 0:
         case 5:
                   pTargetCP = CP_KBL;
                   break;
         case 1:
         case 2:   pTargetCP = CP_UNIX_BALTIC_RIM;
                   break;
         case 3:   pTargetCP = CP_MAC_Respublic;
                   break;

         case 4:   pTargetCP = CP_RST;
                   break;

         case 6:   bWasT772 = TRUE;
         case 7:   pTargetCP = CP_772_774;
                   break;
         case 8:   pTargetCP = WIN_LSPEC0;
                   break;
         case 9:   pTargetCP = WIN_LSPEC_MIX;
                   break;
         case 10:  pTargetCP = WIN_LEMA;
                   break;
         case 11:  pTargetCP = WIN_TILDE;
                   break;
         case 12:  pTargetCP = WIN_SIGMA;
                   break;
         case 13:  pTargetCP = LTH_Diplomat;
                   break;
         default:
                  break;
       }
     return TRUE;
    }
   else printf (cIncParm, cTARGET, mparamP2+L);

  return FALSE;
} /* end IsResultCP */


static BOOL WhatKey(mparam)  char *mparam;
{ char c = (char)tolower(mparam[1]);

  if (c)
     { switch (c)
         { case 'f': if (IsFromKey(mparam+2))      return TRUE;
                     if (IsFull(mparam+2))         return TRUE;
                     break;
           case 't': if (IsToKey(mparam+2))          return TRUE;
                     if (IsResultCP(mparam+2, TRUE)) return TRUE;
                     break;

           case '?': if (mparam[2] == '\0')
                       { bWasHelp = TRUE;
                         return TRUE;
                       }
                     break;

           case 'h': if (stricmp (mparam+2, "elp") == 0)
                       { bWasHelp = TRUE;
                         return TRUE;
                       }
                     break;

           case 'l': if (IsLithuanian(mparam+2))      return TRUE;
                     break;
           case 'c': if (IsCyrillic(mparam+2))        return TRUE;
                     break;
           case 's': if (IsSourceCP(mparam+2))        return TRUE;
                     break;
           case 'r': if (IsResultCP(mparam+2, FALSE)) return TRUE;
                     break;
           case 'n': if (IsNoTrail(mparam+2, FALSE))  return TRUE;
                     break;

           default:
                     break;
         }
     }

  printf ("\nThe parameter '%s' is not recognized", mparam);
  return FALSE;
} /* end WhatKey */

void SetDefLangCP()
{ if      (wSourceOS & ANY_UNIX_CASE)   pSourceCP = defUNIXCP;
  else if ((wSourceOS & MSDOS_CASE) ||
          (wSourceOS & OS2_CASE))       pSourceCP = defDOSCP;
  else if (wSourceOS & MAC_OS_CASE)     pSourceCP = defMACCP;
  else                                  pSourceCP = defSourceCP;

  if (wTargetOS & ANY_UNIX_CASE)        pTargetCP = defUNIXCP;
  else if ((wTargetOS & MSDOS_CASE) ||
          (wTargetOS & OS2_CASE))       pTargetCP = defDOSCP;
  else if (wTargetOS & MAC_OS_CASE)     pTargetCP = defMACCP;
  else                                  pTargetCP = defTargetCP;

  bNeedConv = TRUE;
  bNeedCyrConv = FALSE;
} /* end SetDefLangCP */

BOOL CheckCP()
{ bNeedConv = TRUE;
  bNeedCyrConv = FALSE;

  if (bWasCyr)
     { BOOL bSourceRus = FALSE;

     /* DOS'e, jei 771 ir 772? */
       /* Windows, jei Mixed */
       /* Kitais atvejais - FALSE */
       /* tikrinti ar source & target */
       if (pSourceCP == CP_KBL)             bSourceRus = TRUE;
       else if (pSourceCP == WIN_LSPEC_MIX) bSourceRus = TRUE;
       else if (pSourceCP == CP_772_774)
            if (bWasS772)                    bSourceRus = TRUE;

       if (bSourceRus)
          { if (pTargetCP == CP_KBL)             bNeedCyrConv = TRUE;
            else if (pTargetCP == WIN_LSPEC_MIX) bNeedCyrConv = TRUE;
            else if (pTargetCP == CP_772_774)
                if (bWasT772)                    bNeedCyrConv = TRUE;
          }
       return bNeedCyrConv;
     }
  return TRUE;
}

void OutputHelp()
{ printf ("\nThe use of XEOL:\n"
  "\tXEOL  [options] sourceFN targetFN\n\n"
  "Here:   [...] mean that the object can omitted;\n"
  "Program understunds such keys:\n"
  "'-f[rom]:OS' and '-t[o]:OS' - to define OS'es.\n"
  "\tAcceptable OS's values: DOS, UNIX, OS2 or MAC;\n"
  "'-s[ourceCP]:CP' and -'r[esultCP]:CP' - to define code tables.\n"
  "\tAcceptable CP's values: KBL, 771, 770, 772, 774, UNIX, RIM, MAC,\n"
  "\tSPECTRUM, MIXED, LEMA, TILDE, SIGMA and DIPLOMAT;\n"
  "'-Lithuanian' or '-LTH' - need to do the Lithuanian conversion;\n"
  "'-Cyrrilic' or '-Cyr' - need to do the Russian conversion;\n"
  "'-full' - don't erase 'bolding' formatting;\n"
  "'-notrail' - don't erase blanks before line breaks.\n\n");
} /* end OutputHelp */

/* Main program entry */

static struct stat stbFrom;
static struct stat stbTo;
static char cInputFN[272];
static char cOutputFN[272];

static char no_acc[] = "No access to file %s\n";
static char no_acc_dir[] = "You specified not existing directory in file %s\n";
static char no_dir[] = "You specified the name of directory: %s\n";

BOOL TryRussianConvert(cptr) char *cptr;
{ register short int iIndex = 0;
  if (bNeedCyrConv)
     {
        while (pCyrSourceCP[iIndex])
           { if (pCyrSourceCP[iIndex] == *cptr)
               { if (iIndex == 18) *cptr = 'y'; /* perklojam lemos durnystes */
                 else if (iIndex == 18) *cptr = 'Y';
                 else *cptr = pCyrTargetCP[iIndex] ;
                 return TRUE;
               }
             ++iIndex;
           }

     }
   return FALSE;
} /* TryRussianConvert */

BOOL LTHConvert(cptr) char *cptr;
{ register short int iIndex = 0;

  while (pSourceCP[iIndex])
     { if (pSourceCP[iIndex] == *cptr)
         { if (iIndex == 18) *cptr = 'y';   /* perklojam lemos durnystes */
           else if (iIndex == 18) *cptr = 'Y';
           else *cptr = pTargetCP[iIndex] ;
           return TRUE;
         }
       ++iIndex;
     }
  return FALSE;
} /* end LTHConvert */

void CodeConversion(cptr) char *cptr;
{ if (bNeedConv)
     { if (!LTHConvert(cptr)) TryRussianConvert(cptr);
     }
} /* end CodeConversion */

BOOL OutStatErrMsg(bAllErr, fn) BOOL bAllErr; char *fn;
{ switch (errno)
    { case ENOTDIR:
      case EACCES: printf (no_acc_dir, fn);
                   return 1;


      default: if (bAllErr)
                  { printf (no_acc, fn); return 6;
                    return 1;
                  }
               else
                if (errno != ENOENT) /* ar tik DOS ? */
                  { printf ("STAT errno:%d", errno);
                    return 1;
                  }
    }

  return 0;
} /* end OutStatErrMsg */

/* MAIN program entry */
int main(argc, argv) int argc; char **argv;
{ register int count;
  BOOL   bBreak;

  printf ("XEOL v1.5U To port Textual files among platforms\n");
  printf ("(c)1996, Jonas Skendelis. All rights reserved. Not for commercial use.\n");

  -- argc; argv++; count = 0; bBreak = FALSE;
  while (argc > 0)
    { /* patikrinti ir is kelio isskirti sablona */

      if (
#ifdef _DOS_CASE
             /* In case of MS-DOS '/' can be used as alternate char
                to identify the option
             */
             ((*argv)[0] == '/') ||
#endif
             ((*argv)[0] == '-'))
         { /* Rakto analize */
           if (WhatKey(*argv) == FALSE)
             { OutputHelp();
               return 4;
             }
         }
      else
       { if (count == 0) strcpy (cInputFN, *argv);
         else if (count == 1) strcpy (cOutputFN, *argv);
         ++count;
       }
      --argc; argv++;
    }  /* arg check */

 if (count > 2)
    { printf ("Too many files in the command line\n");
      OutputHelp();
      return 4;
    }
 else if (count < 2)
    { printf ("Must be exactly 2 filenames in the command line\n");
      OutputHelp();
      return 4;
    }

  if (bWasHelp) OutputHelp();

  if (stat(cInputFN, &stbFrom) < 0)
     { OutStatErrMsg(TRUE, cInputFN);
       return 6;
     }

  if (stbFrom.st_mode & S_IFDIR)
     { printf (no_dir, cInputFN); return 6;
     }

  if (stat(cOutputFN, &stbTo) < 0)
     { if (OutStatErrMsg(FALSE, cOutputFN)) return 6;
     }
  else
    { /* Failas egzistuoja */
      printf ("Output exists\n");
      remove(cOutputFN);
    }

  if (stbTo.st_mode & S_IFDIR)
     { printf (no_dir, cOutputFN);
       return 6;

       /* O gal failo nera? */
     }

  bBreak = TRUE;
  if (bWasLith)
    if ((bWasfCP == FALSE) && (bWasrCP == FALSE))
      { bBreak = FALSE;
        SetDefLangCP();
      }


  if ((bWasfCP != FALSE) || (bWasrCP != FALSE))
     { /* Check correctness of CP compatibility with platforms */

       if (!CheckCP())
         { printf ("\nThe imposible combination of the code tables for convertion\n"
                     "The conversion is turned off\n");
         }
     }

  return (Perform());
} /* end main */

static char NO_SUCH[] = "No such file%s.\n";
static char TOO_MANY_OF[]= "Too many files are opened.\n";

void OutOpenError(bCreat) int bCreat;
{ switch (errno)
  { case EACCES: printf ("No such path or %s denies permission\n",
                          (bCreat == FALSE) ? "opening" : "creating");
                 break;

    case EMFILE: printf (TOO_MANY_OF); break;
    case ENOENT: printf (NO_SUCH, ""); break;

#ifndef _DOS_CASE
    case ENFILE: printf ("The system file table is full.\n");
                 break;

    case EAGAIN: printf ("The file is locked by other process.\n");
                 break;
    case EISDIR: printf (no_dir, "");              break;
    case EROFS:  printf ("The attempt to create file on read-only device.\n");
                 break;
#endif
  }
} /* end OutOpenError */

int Perform()
 { int k;
  unsigned int count;
  BOOL   bBreak;

  if ((k = TryAllocBuffs(maxbufsize))) return k;

  bBreak = FALSE;
  printf ("\nProcessing '%s' file ..", cInputFN);

  hSource =
#ifdef _CASE_OLD_UNIX
     open(cInputFN, UU_RDONLY);
#else
     open(cInputFN, O_BINARY | O_RDONLY, S_IREAD);
#endif

  if (hSource == - 1)
     { printf ("\nError opening file %s.\n", cInputFN);
       OutOpenError(0);
       return 8;
     }
   else
    { hTarget =
#ifdef _CASE_OLD_UNIX
      creat (cOutputFN, stbFrom.st_mode);
#else
      open(cOutputFN, O_CREAT | O_BINARY | O_WRONLY, S_IWRITE);
#endif
      if (hTarget == -1)
        { printf ("\nError creating file %s.\n", cOutputFN);
          OutOpenError(1);
          return 8;
        }
      else
        { bytestoread = bytesafterread =  maxbufsize;
          numread = 0; outquant = 0; bWasCRLF = FALSE;

          while ((numread+bytestoread) == maxbufsize)
            { putchar('.');
              bytesafterread = read(hSource, readbuf, maxbufsize);
              if (bytesafterread == -1)
                { printf("\nError reading the file %s\n", cInputFN);
                  return 8;
                }
               numread += bytesafterread;

               for (count = 0; count < numread; count++)
                 { if ((bWasFull != FALSE) &&
                       (readbuf[count] == BACKSPACE_CHAR))
                     { if (wSourceOS & ANY_UNIX_CASE)
                         { /* Doesn not to eat the line breaks */
                           if ((outquant != 0) && (bWasCRLF == FALSE))
                             --outquant;
                           else
                             { /* Such situation can be at file begin, only. */
                               if (OutByte(readbuf[count]))
                                 { err_wri_tail:
                                   printf ("\nError writing to file:%s", cOutputFN);
                                   return 16;
                                 }
                             }
                         }
                       else  if (OutByte(readbuf[count])) goto err_wri_tail;
                       bWasCRLF = FALSE;
                     }
                   else
                     { if (readbuf[count] == LF_CHAR)
                         { if ((wSourceOS & MAC_OS_CASE) == 0)
                             { if (!bWasNoTrail) EatTrailingWhiteSpace();
                               if (OutputLineBreak()) goto err_wri_tail;
                               bWasCRLF = TRUE;
                             }
                         }
                       else
                         if (readbuf[count] == CR_CHAR)
                           { if (wSourceOS & MAC_OS_CASE)
                               { if (!bWasNoTrail) EatTrailingWhiteSpace();
                                 if (OutputLineBreak()) goto err_wri_tail;
                                 bWasCRLF = TRUE;
                               }
                           }
                       else
                         { /* Any other byte is sending to output */
                           if (bNeedConv)  CodeConversion(&readbuf[count]);
                           if (OutByte(readbuf[count]))
                                       goto err_wri_tail;

                           bWasCRLF = FALSE;
                         }
                     }
                 } /* end for */
               bytestoread = numread; numread = 0;
            } /* end while */

          if (PurgeOutput()) goto err_wri_tail;

          close(hTarget);
        } /* if creat */
      close(hSource);
    } /* if open */

  freeBuffs();
  printf ("\nAll done.\n");
  return 0;
} /* end Perform */

